@@ -84,12 +84,12 @@ class ApoJWT():
                     if self.__exception_handler is None:
                         raise jwt.exceptions.InvalidTokenError(f"JWT is missing - Token Finder failed: {e}")
                     else:
-                        kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg=f"JWT is missing - Token Finder failed: {e}")
+                        kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg=f"JWT is missing - Token Finder failed: {e}")
                         self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                         self.__exception_handler(*args, **kwargs)
                         raise NotImplementedError("ApoJWT Exception Handler must not return control")
                 token_dict = self.__decode(token, *args, **kwargs)
-                kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"], token_permissions=token_dict["perms"])
                 self.__form_kwargs(fn, kwarg_dict, kwargs)
                 return await fn(*args, **kwargs)
             return wrapper
@@ -104,12 +104,12 @@ class ApoJWT():
                     if self.__exception_handler is None:
                         raise jwt.exceptions.InvalidTokenError(f"JWT is missing - Token Finder failed: {e}")
                     else:
-                        kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg=f"JWT is missing - Token Finder failed: {e}")
+                        kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg=f"JWT is missing - Token Finder failed: {e}")
                         self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                         self.__exception_handler(*args, **kwargs)
                         raise NotImplementedError("ApoJWT Exception Handler must not return control")
                 token_dict = self.__decode(token, *args, **kwargs)
-                kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"], token_permissions=token_dict["perms"])
                 self.__form_kwargs(fn, kwarg_dict, kwargs)
                 return fn(*args, **kwargs)
             return wrapper
@@ -118,7 +118,7 @@ class ApoJWT():
     def permission_required(self, permission: str):
         """Verifies a JWT and ensures it contains the correct permission for the resource
     
-        permission: permission string that will be fed into permission_formatter if defined
+        permission: permission string
 
         Raises an exception if any claims are invalid
             - expired token
@@ -138,27 +138,26 @@ class ApoJWT():
                         if self.__exception_handler is None:
                             raise jwt.exceptions.InvalidTokenError(f"JWT is missing - Token Finder failed: {e}")
                         else:
-                            kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg=f"JWT is missing - Token Finder failed: {e}")
+                            kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg=f"JWT is missing - Token Finder failed: {e}")
                             self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                             self.__exception_handler(*args, **kwargs)
                             raise NotImplementedError("ApoJWT Exception Handler must not return control")
                     token_dict = self.__decode(token, *args, **kwargs)
                     decoded_permissions = token_dict["perms"]
                     if self.__admin_perm is not None and self.__admin_perm in decoded_permissions:
-                        kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                        kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"], token_permissions=token_dict["perms"])
                         self.__form_kwargs(fn, kwarg_dict, kwargs)
                         return await fn(*args, **kwargs)
-
-                    formatted_perm = permission if self.__permission_formatter is None else self.__permission_formatter(permission=permission, *args, **kwargs)                        
-                    if formatted_perm not in decoded_permissions:
+                        
+                    if permission not in decoded_permissions:
                         if self.__exception_handler is None:
                             raise jwt.exceptions.InvalidTokenError(f"403: JWT is not authorized for this action")
                         else:
-                            kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=403, msg="JWT is not authorized for this action")
+                            kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=403, msg="JWT is not authorized for this action")
                             self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                             self.__exception_handler(*args, **kwargs)
                             raise NotImplementedError("ApoJWT Exception Handler must not return control")
-                    kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                    kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"], token_permissions=token_dict["perms"])
                     self.__form_kwargs(fn, kwarg_dict, kwargs)
                     return await fn(*args, **kwargs)
                 return wrapper
@@ -173,27 +172,26 @@ class ApoJWT():
                         if self.__exception_handler is None:
                             raise jwt.exceptions.InvalidTokenError(f"JWT is missing - Token Finder failed: {e}")
                         else:
-                            kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg=f"JWT is missing - Token Finder failed: {e}")
+                            kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg=f"JWT is missing - Token Finder failed: {e}")
                             self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                             self.__exception_handler(*args, **kwargs)
                             raise NotImplementedError("ApoJWT Exception Handler must not return control")
                     token_dict = self.__decode(token, *args, **kwargs)
                     decoded_permissions = token_dict["perms"]
                     if self.__admin_perm is not None and self.__admin_perm in decoded_permissions:
-                        kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                        kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"], token_permissions=token_dict["perms"])
                         self.__form_kwargs(fn, kwarg_dict, kwargs)
                         return fn(*args, **kwargs)
 
-                    formatted_perm = permission if self.__permission_formatter is None else self.__permission_formatter(permission, *args, **kwargs)
-                    if formatted_perm not in decoded_permissions:
+                    if permission not in decoded_permissions:
                         if self.__exception_handler is None:
                             raise jwt.exceptions.InvalidTokenError(f"403: JWT is not authorized for this action")
                         else:
-                            kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=403, msg="JWT is not authorized for this action", *args, **kwargs)
+                            kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=403, msg="JWT is not authorized for this action", *args, **kwargs)
                             self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                             self.__exception_handler(*args, **kwargs)
                             raise NotImplementedError("ApoJWT Exception Handler must not return control")
-                    kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                    kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"], token_permissions=token_dict["perms"])
                     self.__form_kwargs(fn, kwarg_dict, kwargs)
                     return fn(*args, **kwargs)
                 return wrapper
@@ -215,7 +213,7 @@ class ApoJWT():
                     if self.__exception_handler is None:
                         raise jwt.exceptions.InvalidTokenError(f"Refresh JWT is missing - Refresh Token Finder failed: {e}")
                     else:
-                        kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg=f"Refresh JWT is missing - Refresh Token Finder failed: {e}")
+                        kwarg_dict = dict(refresh_data=dict(), code=401, msg=f"Refresh JWT is missing - Refresh Token Finder failed: {e}")
                         self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                         self.__exception_handler(*args, **kwargs)
                         raise NotImplementedError("ApoJWT Exception Handler must not return control")
@@ -237,7 +235,7 @@ class ApoJWT():
                     if self.__exception_handler is None:
                         raise jwt.exceptions.InvalidTokenError(f"Refresh JWT is missing - Refresh Token Finder failed: {e}")
                     else:
-                        kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg=f"Refresh JWT is missing - Refresh Token Finder failed: {e}")
+                        kwarg_dict = dict(refresh_data=dict(), code=401, msg=f"Refresh JWT is missing - Refresh Token Finder failed: {e}")
                         self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                         self.__exception_handler(*args, **kwargs)
                         raise NotImplementedError("ApoJWT Exception Handler must not return control")
@@ -304,7 +302,7 @@ class ApoJWT():
             if self.__exception_handler is None:
                 raise jwt.exceptions.InvalidSignatureError(f"401: JWT signature is invalid")
             else:
-                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg="JWT signature is invalid")
+                kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg="JWT signature is invalid")
                 self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                 self.__exception_handler(*args, **kwargs)
                 raise not_implemented
@@ -312,7 +310,7 @@ class ApoJWT():
             if self.__exception_handler is None:
                 raise jwt.exceptions.ExpiredSignatureError(f"403: JWT signature has expired")
             else:
-                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=403, msg="JWT signature has expired")
+                kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=403, msg="JWT signature has expired")
                 self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                 self.__exception_handler(*args, **kwargs)
                 raise not_implemented
@@ -320,7 +318,7 @@ class ApoJWT():
             if self.__exception_handler is None:
                 raise jwt.exceptions.InvalidIssuerError(f"401: JWT issuer is invalid")
             else:
-                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg="JWT issuer is invalid")
+                kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg="JWT issuer is invalid")
                 self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                 self.__exception_handler(*args, **kwargs)
                 raise not_implemented
@@ -328,7 +326,7 @@ class ApoJWT():
             if self.__exception_handler is None:
                 raise jwt.exceptions.InvalidAudienceError(f"401: JWT audience is invalid")
             else:
-                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg="JWT audience is invalid")
+                kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg="JWT audience is invalid")
                 self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                 self.__exception_handler(*args, **kwargs)
                 raise not_implemented
@@ -336,7 +334,7 @@ class ApoJWT():
             if self.__exception_handler is None:
                 raise jwt.exceptions.InvalidTokenError(f"401: JWT is invalid")
             else:
-                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, msg=f"JWT is invalid: {e}")
+                kwarg_dict = dict(token_data=dict(), token_subject="", token_permissions=list(), access_token="", code=401, msg=f"JWT is invalid: {e}")
                 self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
                 self.__exception_handler(*args, **kwargs)
                 raise not_implemented
